package crowdmodel;

import repast.simphony.context.Context;
import repast.simphony.context.space.graph.NetworkFactoryFinder;
import repast.simphony.dataLoader.SubContextCreator;

/**
 * The agentContext creator defines the creation of a subcontext for the crowd simulation.
 * Here this involves a subcontext for the crowd agents to define different 
 * network projections just for those agents (and thus nodes) 
 * This reason for creating a different context for it is that it literally discerns between the
 * social and the physical world but also for display purposes, as I can choose to display the 
 * agent structures on its own.
 * 
 * @author Nanda Wijermans
 *
 */

public class AgentContextCreator extends SubContextCreator{

	/**
	 * Creation of Network projections
	 * 1) Social network: the description of the network structure in terms of
	 * who you are coming with to the crowd event. This depend on the scenario.
	 * 2) Leadership network: a representation 
	 * 3) Clusters of behaviour : the group-level measure used in this simulation
	 */

	
@Override
	public Context build(Context socialContext) {

		NetworkFactoryFinder.createNetworkFactory(null).createNetwork(
				"Social Network", socialContext, true);
		
		NetworkFactoryFinder.createNetworkFactory(null).createNetwork(
				"Leadership Network", socialContext, true);
	
		NetworkFactoryFinder.createNetworkFactory(null).createNetwork(
				"Behaviour Network", socialContext, false);
		
		return socialContext; 
	}
	
}
