package crowdmodel;

import repast.simphony.annotate.AgentAnnot;
import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.util.ContextUtils;

/**
 *  The abstract class of all objects and classes in the CrowdModel
 *  An object is an environment object or an agent (individual)
 *  
 *  @author Nanda Wijermans
 */

@AgentAnnot(displayName = "CObject")

public abstract class CObject{
	public int d_id;				// identity of object
	
	public Context<CObject> d_world_context;

public void CObject(int id, Context worldContext){
	//needs to be overridden
	d_id = id;
	d_world_context = worldContext; 
}
	
	
/** 
 * Step method describes what this object will do in the given time step
 * This is to be defined in every class that is implementing this abstract class
 * 
 *  The @scheduleMethod indicates to the Scheduler (Repast)
 *  that the step method needs is to be scheduled starting at
 *  tick 1 with an interval of 1 tick. 
 *  Thus: the step starts at 1, and recurs at 2,3,4,...etc
 */

//@ScheduledMethod(start = 0, interval = 1, priority = 1.0)
	public void step(){
		System.out.println("Cobject.step:" + d_id);
		// Override by subclasses 
	}
   
/**
 *  Delete the CObject 
 *  In repast terms this means removing the Object from the context
 */
	public void delete(){
		// Get the context in which the agent resides.
		Context context = ContextUtils.getContext(this);
		// Remove the agent from the context if the context is not empty
		if (context.size() > 1)
			context.remove(this);
		else // Otherwise if the context is empty, end the simulation
			RunEnvironment.getInstance().endRun();
	}	

	
/**
 * Returns the id of the CObject. This is an unique number (integer) 
 * that identifies this object
 * 
 * @return the unique ID of this CObject	
 */
	public int getID(){
		return d_id;
	}
	
}
