package crowdmodel;

import java.awt.Polygon;

import crowdmodel.environment.Area;
import crowdmodel.environment.CellObject;
import crowdmodel.utils.MiscSingleton;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.environment.RunState;
import repast.simphony.engine.schedule.ScheduleParameters;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.parameter.Parameters;
import repast.simphony.context.Context;

/**
 * Class that in initialises CROSS (crowd simulation)
 * 
 * @author Nanda Wijermans
 *
 */

public class InitCrowdSim {
	
	/*  
	 * The environment parameters contain the user-editable values that appear in the GUI.
	 * Retrieves the parameters p. 
	 */
	Parameters p = RunEnvironment.getInstance().getParameters();

	int d_worldWidth = (Integer) p.getValue("worldWidth");
	int d_worldHeight = (Integer) p.getValue("worldHeight");
	
	int d_crowdsize = (Integer)p.getValue("crowdsize");
	int d_numLeaders = (Integer)p.getValue("nrLeaders");
	int d_numWLeaders = (Integer)p.getValue("warLeaders") * d_numLeaders;	// percentage of leaders that direct in an aggressive way
	int d_numPLeaders = d_numLeaders - d_numWLeaders;   // percentage of leaders that direct in a peaceful way	
	
	Context d_context = RunState.getInstance().getMasterContext();
	
	/**
	 * Constructor of initialisation of CROSS
	 * Retrieves the information from GUI necessary to build 
	 * the relevant scenario 
	 * 
	 */
	

	public InitCrowdSim(){
		//dunno what to put here, think think...
		
	}



@ScheduledMethod(start = 0, interval = 0) 
	public void initialise(){
		initWorld();
		defineAreas();
	}
	
	private void initWorld(){
		/* 
		 * Populate the patch grid with the terrain and specific festival areas
		 * Iterate over the dimensions of the patch grid 
		 */	
		Context context = RunState.getInstance().getMasterContext();
		
		for (int i = 0; i < d_worldWidth; ++i){
			for (int j = 0; j < d_worldHeight; ++j){
				CellObject cell = new CellObject(d_context, i, j, MiscSingleton.getInstance().nextID()); // create an empty terrain cell
			}
		}
	}
	
	private void defineAreas(){
		
		Polygon stagePolygon = new Polygon(Constants.stageX, Constants.stageY, 4);
		Polygon barPolygon = new Polygon(Constants.barX, Constants.barY, 4);
		Polygon toiletPolygon = new Polygon(Constants.toiletX, Constants.toiletY, 4);
		
		/* stage */
		Area d_stage = new Area(d_context, "stage", stagePolygon, false);
		d_stage.addPoi(Constants.POI_STAGE);
 		d_stage.setPOIs();
 		
		/* bar */
		Area d_bar = new Area(d_context, "bar", barPolygon, false);
		d_bar.addPoi(Constants.POI_BAR);
		d_bar.addPoi(Constants.POI_BAR1);
		d_bar.addPoi(Constants.POI_BAR2);
		d_bar.setPOIs();
		
		/* toilet */
		Area d_toilet = new Area(d_context, "toilet", toiletPolygon, false);
		d_toilet.addPoi(Constants.POI_TOILET);
		d_toilet.addPoi(Constants.POI_TOILET1);
		d_toilet.addPoi(Constants.POI_TOILET2);
 		d_toilet.setPOIs();
 		
	}
	
}
