package crowdmodel.agents;

import java.util.HashMap;

import crowdmodel.Constants;

/**
 * Describes a specific type of Fact (MemoryElement)
 * related to the behaviours an agent has.
 * 
 * A behaviour fact typically contains the expectations
 * an agent has in executing this behaviour related
 * to the 4 goals an individual has 
 * 
 * See thesis: chapter 3 (theoretical model) and 5 (computational model)
 * Concerning memory  section 3.3.2 and section 5.2.2.
 * 
 * @author Nanda Wijermans
 *
 */
public class BehaviourFact extends Fact{

	/* Expectancies in fulfilling a goal when executing this behaviour */	
	HashMap<String, Double> d_expectancies;
	
	
	/**
	 * Constructor creates a behaviour fact 
	 * given the initial expectation setting
	 * 
	 */
	public BehaviourFact(String name, double expIdentity, double expSocial, double expSafety, double expSubs){
		d_name = name;
		d_expectancies = new HashMap<String, Double>();
		d_expectancies.put(Constants.GOAL_NAME_IDENTITY, expIdentity);
		d_expectancies.put(Constants.GOAL_NAME_SOCIAL, expSocial);
		d_expectancies.put(Constants.GOAL_NAME_SAFETY, expSafety);
		d_expectancies.put(Constants.GOAL_NAME_SUBSISTENCE, expSubs);
	}
	
 	/**
	 * Sets an expectancy value of a specified goal
	 */
	public void setExpectancy(String goalName, double expectancy){
		d_expectancies.put(goalName, expectancy);
	}
		
	
	/**
	 * Returns the expectancy of the goal asked for
	 * this is the expectancy an agent has in fulfilling
	 * the desired goal when executing this behaviour
	 */
	public double getExpectancy(String goalName){
		return (double) d_expectancies.get(goalName);
	}


}
