package crowdmodel.agents;

/** 
 * Fact is a type of memoryElement. 
 * Memory has 3 types of elements: Facts, Rules and Goals
 * A fact defines a unit of declarative knowledge (a chunk)
 * 
 * @author Nanda Wijermans
 *
 */

public abstract class Fact extends MemoryElement {


	
	
	/**
	 * Returns the retrievalTime of this element
	 * The behaviour selection process is time dependent,
	 * the retrieval of a fact in memory takes time, which is
	 * here calculated based upon the activation level this element has.
	 *
	 * activation : [-infinity,infinity] following an natural logaritm function<br>
	 * we limit this by [-6,6] as the activation is a ln(x) and given the range of the
	 * average memoryElement lies more or less between this range. So the finesses between
	 * this range should be reflected in terms of time, larger or smaller numbers are not relevant
	 * as it either approaches the max or min amount of retrieval time.
	 * 
	 * The translation between the activation and the retrieval time follows an sigmoid function
	 * As this has the characteristics (steep decrease in time, as the activation time rises, however
	 * also follows a smoothed threshold when activation is lower than zero.
	 * 
	 *
	 */
	public double getRetrievalTime() {
		
		//(1 - (1 / (1 + Math.exp(-((d_density * (12 / Constants.MAX_DENSITY))-6)))));
		
		/* correct for the extreme values bigger than 6 and smaller than -6 */ 
		double activation = (d_activation > 6 ? 6 : d_activation);
		activation = (d_activation < -6 ? -6 : d_activation);
		
		double sigmoidOutcome =  1 / (1 + Math.exp(-(d_activation)));
		double rtime = 1 - sigmoidOutcome;
				
		return rtime;
	}
}
