package crowdmodel.agents;


/**
 * Describes a specific type of Fact (MemoryElement)
 * related to a person (or a group).
 * 
 * Indicates how this person is perceived 
 * - leader or not
 * - ingroup | outgroup | unknown
 * 
 * //TODO refer to pages in thesis
 * 
 * @author Nanda Wijermans
 *
 */
public class PersonFact extends Fact {

	/* 
	 * group membership 
	 */
	
	
	boolean d_leader;    // whether this person is perceived as a leader
	int d_membership;    // 0 = ingroup; 1 = outgroup; 2 = unknown
	boolean d_specificTie; // indicates whether there is a connection with this person specificly or a general conception of a member of the group
	
	int d_personID;		// the ID of the person the personFact is related to
	
	/**
	 * The constructor creates an PersonFact and 
	 * sets the initial settings
	 */
	public PersonFact(String name, boolean specific, boolean leader, int membership, int idPerson){
		d_name = name;
		d_specificTie = specific;
		d_leader = leader;
		d_membership = membership;
		d_personID = idPerson;
	}
	
	
	/**
	 * Sets the variable d_leader that represents whether this
	 * person is perceived as a leader or not by this agent
	 */
	public void setPerceivedLeadership(boolean leader){
		d_leader = leader;
	}
	
	/**
	 * Returns whether the person this fact relates to is
	 * viewed as a leader by this agent
	 */
	public boolean getPerceivedLeadership(){
		return d_leader;
	}
	
	/**
	 * Sets how the membership is perceived of the related person
	 * by this agent
	 */
	public void setPerceivedMembership(int membership){
		d_membership = membership;
	}
	
	/**
	 * Returns whether the person this fact relates to 
	 * is viewd as ingroup (0), outgroup(1), or unknown (2)
	 * group-member
	 */
	public int getPerceivedMembership(){
		return d_membership;
	}
	
	/**
	 * Returns the name of the fact.
	 * Name = "friend" + id(of friend)
	 */
	public String getName(){
		return d_name;
	}
	
	/** 
	 * Returns the ID of the person
	 * this personFact is about
	 * @return
	 */
	public int getPersonID(){
		return d_personID;
	}
}
