package crowdmodel.agents;


/**
 * 
 * Sensor is one of the architecture main types.
 * Architecture consists out of a sensor, motor, memory and physiology part.
 * All there is to describe behaviour, influences factors, processes, characteristics etc. 
 * need to be described in either one of the (derived) main parts of the architecture
 * 
 * Sensor: describes the sensing part of an individual
 * In representing the sensors of the human cognitive system
 * contraints are inflicted upon perception by being embodied (ie having a body)
 * For instance: the range of eyes | ears affect what is perceived and more important
 * what is not perceived describes the action/execution of an individual.  
 * 
 * @author Nanda Wijermans
 * 
 */

public abstract class Sensor {

	
	/* perception range variables */ 
	private int width;   	// width of perception in degrees, default 180 degrees
	private int depth;  	// depth of perception
	
	protected Sensor(int width, int depth) {
		this.width = width;
		this.depth = depth; 
	}
	
	/* retrieves info from the patches that are within the scope of perception */
	public void getInfo(){
		
	}
	
	protected void setWidth(int width){
		this.width = width;
	}
	public int getWidth(){
		return width;
	}
	protected void setDepth(int depth){
		this.depth = depth;
	}
	public int getDepth(){
		return depth;
	}
}
