package crowdmodel.environment;

import repast.simphony.context.Context;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.grid.Grid;
import repast.simphony.util.ContextUtils;
import repast.simphony.valueLayer.GridValueLayer;
import crowdmodel.CObject;

/**
 * Describes environmental objects and their location on the grid 
 * An object is always situated on one gridCell. Can be part of a 
 * larger object, however this is then a set cellObjects.
 * 
 * @author Nanda Wijermans
 *
 */

public class CellObject extends CObject{

	public static final int NOOBJECTS = 0;
	public static final int STAGE = 1;
	public static final int BAR = 2;
	public static final int TOILET = 3;
	public static final int POI = 4;
	
	private String d_name = "terrain";
	public boolean d_walkable = true;  // boolean to indicate a cell to be walkable or not */
	
	final int d_x, d_y;    //for now at least, the objects aren't supposed to move.
	
	
	/**
	 * Constructor of CellObject
	 * Creates the cell object
	 */
	public CellObject(Context<CObject> worldcontext, int x, int y, int id){
		d_x = x;
		d_y = y;
		d_id = id;  
		
		worldcontext.add(this);   
		
		Grid grid = (Grid) worldcontext.getProjection("Simple Grid");
		ContinuousSpace space = (ContinuousSpace) worldcontext.getProjection("Continuous Space");
		
		/* 
		 * move the areaCell to its position on the patch grid 
		 * and to its areaCell on the continuous space 
		 */
		grid.moveTo(this, x, y);
		space.moveTo(this, x, y, 0);	
		
		GridValueLayer v = (GridValueLayer) worldcontext.getValueLayer("Terrain");
		v.set(NOOBJECTS, grid.getLocation(this).toIntArray(null));
	}
	
	/**
	 * Returns the x-coordinate of the cellObject's position
	 */
	public int getX(){
		return d_x;
	}
	

	/**
	 * Returns the y-coordinate of the cellObject's position
	 */
	public int getY(){
		return d_y;
	}
	

	/**
	 * Sets the name-label of the cellObject
	 */
	public void setName(String name){
		d_name = name;
	}

	/**
	 * Returns the name-label of the cellObject
	 */
	public String getName(){
		return d_name;
	}

		
	/**
	 * Sets the value of the cellObject in the valueLayer
	 * Where this value corresponds with the environment Object it
	 * belongs to as the valueLayer uses this value to visualise
	 * these objects on the display.
	 * 
	 * NOTE: This setValue() is only used when the context is created.
	 * As the context doesn't exist yet is cannot be addressed by another class
	 * so the context is being passed as a parameter
	 * 
	 * I do not prefer to use it this way, still finding a way to avoid using
	 * passing the context as a parameter
	 */
	public void setValue(Context<CObject> worldcontext){
		
		Grid grid = (Grid)worldcontext.getProjection("Simple Grid");;
		GridValueLayer v = (GridValueLayer)worldcontext.getValueLayer("Terrain");

		if (d_name == "stage")
			v.set(STAGE, grid.getLocation(this).toIntArray(null));
		else if (d_name == "bar") 
			v.set(BAR, grid.getLocation(this).toIntArray(null));
		else if (d_name == "toilet")
			v.set(TOILET, grid.getLocation(this).toIntArray(null));
		else if (d_name == "poi")
			v.set(POI, grid.getLocation(this).toIntArray(null));
		else
			System.out.println("CellObject.setValue() wants to set a type of object that doesn't exists");
			//TODO syst.err
	}

	
	/**
	 * Sets the value of the cellObject in the valueLayer
	 * Where this value corresponds with the environment Object it
	 * belongs to as the valueLayer uses this value to visualise
	 * these objects on the display.
	 */
	public void setValue(){
		Grid grid = (Grid) ContextUtils.getContext(this).getProjection("Simple Grid");
		GridValueLayer v = (GridValueLayer)ContextUtils.getContext(this).getValueLayer("Terrain");

		if (d_name == "stage")
			v.set(STAGE, grid.getLocation(this).toIntArray(null));
		else if (d_name == "bar")
			v.set(BAR, grid.getLocation(this).toIntArray(null));	
		else if (d_name == "toilet")
			v.set(TOILET, grid.getLocation(this).toIntArray(null));
		else if (d_name == "poi")
			v.set(POI, grid.getLocation(this).toIntArray(null));
		else
			System.out.println("CellObject.setValue() wants to set a type of object that doesn't exists");
	}
	
	
	
	
	/**
	 * Sets the CellObject as being walkable or not
	 */
   public void setWalkable(boolean walkable){
		d_walkable = walkable;
	}

	/**
	 * Returns whether the CellObject is walkable 
	 */
	public boolean getWalkable(){
		return d_walkable;
	}
}
