package crowdmodel.gui;

import javax.swing.AbstractButton;
import javax.swing.Box;
import javax.swing.ButtonModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JComboBox;
import javax.swing.JCheckBox;
import javax.swing.JSlider;
import javax.swing.JSeparator;
import javax.swing.BoxLayout;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import java.awt.Color;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Dictionary;
import java.util.HashMap;
import java.util.Hashtable;

//import org.eclipse.swt.layout.GridData;
import com.jgoodies.forms.layout.*;
import com.jgoodies.forms.builder.*;

import crowdmodel.Constants;

import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.environment.RunEnvironmentBuilder;
import repast.simphony.scenario.ModelInitializer;
import repast.simphony.scenario.Scenario;
import repast.simphony.ui.RSApplication;

/**
 * The custom made user interface tab: user panel
 * is defined/designed here.
 * 
 * Everything needs to be defined here, the components, layout,
 * ActionListeners, update of the GUI components and simuation
 * parameters etc. 
 * 
 * The only thing that Repast does for you is provide a method
 * that gets/sets the simulation parameters
 * 
 * @author Nanda Wijermans
 *
 */

public class UserPanelInitializer implements ModelInitializer{

	/*
	 * Names to avoid typo problems and thus errors
	 */
	
	public static final String CBidentity = "CBidentity";
	public static final String CBsocial = "CBsocial";
	public static final String CBsafety = "CBsafety";
	public static final String CBsubsistence = "CBsubsistence";
	private static final String SLinitID = "SLinitID";
	private static final String SLcustoID = "SLcustoID";
	private static final String SLinitSoc = "SLinitSoc";
	private static final String SLcustoSoc = "SLcustoSoc";
	private static final String SLinitSafe = "SLinitSafe";
	private static final String SLcustoSafe = "SLcustoSafe" ;
	private static final String SLcustoSubs = "SLcustoSubs";
	private static final String ComboIDDist = "ComboIDDist";
	private static final String ComboSocDist = "ComboSocDist";
	private static final String ComboSafeDist = "ComboSafeDist";
	private static final String ComboSubsDist = "ComboSubsDist";
	/*
	 * The GUI-elements
	 * (non-Javadoc)
	 * @see repast.simphony.scenario.ModelInitializer#initialize(repast.simphony.scenario.Scenario, repast.simphony.engine.environment.RunEnvironmentBuilder)
	 */

	JCheckBox d_idCheckb, d_socCheckb, d_safeCheckb, d_subsCheckb;

	JComboBox d_identityPrefCombb, d_socialPrefCombb, d_safetyPrefCombb, d_subsistencePrefCombb;//, d_subsistenceInitCombb;
	
	JSlider d_initIdentity, d_custoIDprefSlider;
	JSlider d_initSocial, d_custoSocPrefSlider;
	JSlider d_initSafety, d_custoSafePrefSlider;
	JSlider d_custoSubsPrefSlider;
	
	ActionListener d_actionListener;
	ChangeListener d_changeListener;
	ItemListener d_itemListener; // not sure if I need this one

	
	/**
	 * Takes care of the creation of my custom made panel while initialising the GUI  
	 */
	public void initialize(Scenario scen, RunEnvironmentBuilder builder) {
		
		if (RSApplication.isGui()){
			RSApplication.getRSApplicationInstance().addCustomUserPanel(
					makeMyPanel());	
			}
		}
	
	
	/**
	 * Defines a custom made panel in which one can design
	 * the user-editable parameters 
	 *  
	 * The GUI allows for setting the settings for the goals
	 * 1) indicate whether the goal should be turned on
	 * 2) the initial satisfaction level of the goal
	 * 3) the preference level of a goal (following a distribution or a customised number)
	 *  
	 */
	
	private JPanel makeMyPanel() {
		myGUIelements();
		
		FormLayout layout = new FormLayout(
			      "3dlu, right:pref, min(6dlu;MIN), 150dlu",  // columns 
			      "top:pref, 8dlu, " +
			      "top:pref, 3dlu, top:pref, 0dlu, top:pref, 3dlu, top:pref, 0dlu, top:pref, 3dlu, " +
			      "top:pref, 8dlu, " +
			      "top:pref, 3dlu, top:pref, 0dlu, top:pref, 3dlu, top:pref, 0dlu, top:pref, 3dlu, " +
			      "top:pref, 8dlu, " +
			      "top:pref, 3dlu, top:pref, 0dlu, top:pref, 3dlu, top:pref, 0dlu, top:pref, 3dlu, " +
			      "top:pref, 8dlu, " +
			      "top:pref, 3dlu, top:pref, 0dlu, top:pref, 3dlu, top:pref, 0dlu, top:pref, 3dlu");           // rows
		
		PanelBuilder builder = new PanelBuilder(layout);
		CellConstraints cc = new CellConstraints();
		builder.addSeparator("Goal settings", cc.xyw(2, 1, 3));
		/* id goal */
		builder.addSeparator("", cc.xyw(2, 3, 3));
		builder.add(d_idCheckb, cc.xy(2, 5));
		builder.addLabel("Initial ID val: ", cc.xy(2, 7));
		builder.add(d_initIdentity, cc.xywh(3, 7, 2, 1));
		builder.addLabel("ID pref dist: ", cc.xy(2, 9));
		builder.add(d_identityPrefCombb, cc.xyw(3, 9, 2));
		builder.addLabel("Custom pref val: ", cc.xy(2, 11));
		builder.add(d_custoIDprefSlider, cc.xyw(3, 11, 2));
		/* social goal */
		builder.addSeparator("", cc.xyw(2, 13, 3));
		builder.add(d_socCheckb, cc.xy(2, 15));
		builder.addLabel("Initial soc val: ", cc.xy(2, 17));
		builder.add(d_initSocial, cc.xywh(3, 17, 2, 1));
		builder.addLabel("Soc pref dist: ", cc.xy(2, 19));
		builder.add(d_socialPrefCombb, cc.xyw(3, 19, 2));
		builder.addLabel("Custom pref val: ", cc.xy(2, 21));
		builder.add(d_custoSocPrefSlider, cc.xyw(3, 21, 2));
		/* safety goal */
		builder.addSeparator("", cc.xyw(2, 23, 3));
		builder.add(d_safeCheckb, cc.xy(2, 25));
		builder.addLabel("Initial safe val: ", cc.xy(2, 27));
		builder.add(d_initSafety, cc.xywh(3, 27, 2, 1));
		builder.addLabel("Safe pref dist: ", cc.xy(2, 29));
		builder.add(d_safetyPrefCombb, cc.xyw(3, 29, 2));
		builder.addLabel("Custom pref val: ", cc.xy(2, 31));
		builder.add(d_custoSafePrefSlider, cc.xyw(3, 31, 2));
		/* subsistence goal */
		builder.addSeparator("", cc.xyw(2, 33, 3));
		builder.add(d_subsCheckb, cc.xy(2, 35));
		
		builder.addLabel("Subs pref dist: ", cc.xy(2, 41));
		builder.add(d_subsistencePrefCombb, cc.xyw(3, 41, 2));
		builder.addLabel("Custom pref val: ", cc.xy(2, 43));
		builder.add(d_custoSubsPrefSlider, cc.xyw(3, 43, 2));
		return builder.getPanel();
	}
	
	
	/**
	 * Defines all the GUI elements that are used displayed in the User Panel.
	 * 
	 */
		
	public void myGUIelements(){

		createListeners();
		
		/*
		 * Goal checkboxes (identity, social, safety subsistence)
		 * to indicate the participation of this goal in the simulation
		 * with listeners to connect the GUIsettings with the simulation settings
		 */
		d_idCheckb = new JCheckBox("identity", true);	
		d_idCheckb.setName(CBidentity);
		d_idCheckb.addItemListener(d_itemListener);
		
		d_socCheckb = new JCheckBox("social", true);
		d_socCheckb.setName(CBsocial);
		d_socCheckb.addItemListener(d_itemListener);	
		
		d_safeCheckb = new JCheckBox("safety", true);
		d_safeCheckb.setName(CBsafety);
		d_safeCheckb.addItemListener(d_itemListener);
		
		d_subsCheckb = new JCheckBox("subsistence", true);		
		d_subsCheckb.setName(CBsubsistence);
		d_subsCheckb.addItemListener(d_itemListener);
		
		String[] distOptions = {Constants.DIST_NORMAL, Constants.DIST_UNIFORM, Constants.DIST_CUSTOM};
		
		d_identityPrefCombb = new JComboBox (distOptions);
		d_identityPrefCombb.setName(ComboIDDist);
		d_identityPrefCombb.addItemListener(d_itemListener);
		
		d_socialPrefCombb = new JComboBox (distOptions);
		d_socialPrefCombb.setName(ComboSocDist);
		d_socialPrefCombb.addItemListener(d_itemListener);
		
		String[] distSafeOptions = {Constants.DIST_NORMAL, Constants.DIST_UNIFORM, Constants.DIST_LEFT_SKEW, Constants.DIST_RIGHT_SKEW, Constants.DIST_CUSTOM};
		d_safetyPrefCombb = new JComboBox (distSafeOptions);
		d_safetyPrefCombb.setName(ComboSafeDist);
		d_safetyPrefCombb.addItemListener(d_itemListener);
		d_safetyPrefCombb.setSelectedIndex(1);
		
		d_subsistencePrefCombb = new JComboBox (distOptions);
		d_subsistencePrefCombb.setName(ComboSubsDist);
		d_subsistencePrefCombb.addItemListener(d_itemListener);
		d_subsistencePrefCombb.setSelectedIndex(2);  // preference initValue (customised)
		
		
		/*
		 * A slider only deals with integers, while I want a value to be displayed
		 * and used between [0,1]. For display I make a dictionary of value-pairs
		 * to make sure to have the right lay-out
		 */		
		Hashtable<Integer, JLabel> slideRange = new Hashtable<Integer, JLabel>();
		slideRange.put(new Integer(0), new JLabel("0.0"));
		slideRange.put(new Integer(2), new JLabel("0.2"));
		slideRange.put(new Integer(4), new JLabel("0.4"));
		slideRange.put(new Integer(6), new JLabel("0.6"));
		slideRange.put(new Integer(8), new JLabel("0.8"));
		slideRange.put(new Integer(10), new JLabel("1.0"));
		
		int def = (int) Math.round(Constants.DEFAULT_INIT_IDENTITY * 10);
		d_initIdentity = new JSlider(JSlider.HORIZONTAL, 0, 10, def); 
		d_initIdentity.setName(SLinitID);
		d_initIdentity.setMajorTickSpacing(2);
		d_initIdentity.setMinorTickSpacing(0);
		d_initIdentity.setPaintTicks(true);
		d_initIdentity.setLabelTable(slideRange);
		d_initIdentity.setPaintLabels(true);
		d_initIdentity.addChangeListener(d_changeListener);
		
		def = (int) Math.round(Constants.DEFAULT_PREF_IDENTITY * 10);
		d_custoIDprefSlider = new JSlider(JSlider.HORIZONTAL, 0, 10, def); 
		d_custoIDprefSlider.setName(SLcustoID);
		d_custoIDprefSlider.setMajorTickSpacing(2);
		d_custoIDprefSlider.setMinorTickSpacing(0);
		d_custoIDprefSlider.setPaintTicks(true);
		d_custoIDprefSlider.setLabelTable(slideRange);
		d_custoIDprefSlider.setPaintLabels(true);
		d_custoIDprefSlider.addChangeListener(d_changeListener);

		def = (int) Math.round(Constants.DEFAULT_INIT_SOCIAL * 10);
		d_initSocial = new JSlider(JSlider.HORIZONTAL, 0, 10, def); 
		d_initSocial.setName(SLinitSoc);
		d_initSocial.setMajorTickSpacing(2);
		d_initSocial.setMinorTickSpacing(0);
		d_initSocial.setPaintTicks(true);
		d_initSocial.setLabelTable(slideRange);
		d_initSocial.setPaintLabels(true);
		d_initSocial.addChangeListener(d_changeListener);
		
		def = (int) Math.round(Constants.DEFAULT_PREF_SOCIAL * 10);
		d_custoSocPrefSlider = new JSlider(JSlider.HORIZONTAL, 0, 10, def); 
		d_custoSocPrefSlider.setName(SLcustoSoc);
		d_custoSocPrefSlider.setMajorTickSpacing(2);
		d_custoSocPrefSlider.setMinorTickSpacing(0);
		d_custoSocPrefSlider.setPaintTicks(true);
		d_custoSocPrefSlider.setLabelTable(slideRange);
		d_custoSocPrefSlider.setPaintLabels(true);
		d_custoSocPrefSlider.addChangeListener(d_changeListener);
		
		def = (int) Math.round(Constants.DEFAULT_INIT_SAFETY * 10);
		d_initSafety = new JSlider(JSlider.HORIZONTAL, 0, 10, def); 
		d_initSafety.setName(SLinitSafe);
		d_initSafety.setMajorTickSpacing(2);
		d_initSafety.setMinorTickSpacing(0);
		d_initSafety.setPaintTicks(true);
		d_initSafety.setLabelTable(slideRange);
		d_initSafety.setPaintLabels(true);
		d_initSafety.addChangeListener(d_changeListener);
		
		def = (int) Math.round(Constants.DEFAULT_PREF_SAFETY * 10);
		d_custoSafePrefSlider = new JSlider(JSlider.HORIZONTAL, 0, 10, def); 
		d_custoSafePrefSlider.setName(SLcustoSafe);
		d_custoSafePrefSlider.setMajorTickSpacing(2);
		d_custoSafePrefSlider.setMinorTickSpacing(0);
		d_custoSafePrefSlider.setPaintTicks(true);
		d_custoSafePrefSlider.setLabelTable(slideRange);
		d_custoSafePrefSlider.setPaintLabels(true);
		d_custoSafePrefSlider.addChangeListener(d_changeListener);
		
		def = (int) Math.round(Constants.DEFAULT_PREF_SUBSISTENCE * 10);
		d_custoSubsPrefSlider = new JSlider(JSlider.HORIZONTAL, 0, 10, def); 
		d_custoSubsPrefSlider.setName(SLcustoSubs);
		d_custoSubsPrefSlider.setMajorTickSpacing(2);
		d_custoSubsPrefSlider.setMinorTickSpacing(0);
		d_custoSubsPrefSlider.setPaintTicks(true);
		d_custoSubsPrefSlider.setLabelTable(slideRange);
		d_custoSubsPrefSlider.setPaintLabels(true);
		d_custoSubsPrefSlider.addChangeListener(d_changeListener);
	}
	
	
	/**
	 * Sets the values coming from the checker-boxes in the Graphical User Interface (GUI)
	 * updates these values in the GUIparams.java which is used by the
	 * simulation to update and set the simulation parameter. 
	 * This represent the connection between the GUI and the simulation.
	 * 
	 * The checker-boxes in the GUI represent the participation of the
	 * role of the possible goals in the agents. They can be switched
	 * on and off here.
	 *
	 * 
	 * @param name
	 * @param selected
	 */
	public void setGoalUsage(String name, Boolean selected){
		
		if (name.equalsIgnoreCase(CBidentity)) {
			GUIparams.getInstance().setIdGoal(selected);
		}
		if (name.equalsIgnoreCase(CBsocial)) {
			GUIparams.getInstance().setSocGoal(selected);
		}
		if (name.equalsIgnoreCase(CBsafety)) {
			GUIparams.getInstance().setSafeGoal(selected);
		}
		if (name.equalsIgnoreCase(CBsubsistence)) {
			GUIparams.getInstance().setSubsGoal(selected);
			}
	}
	 
	/**
	 * Sets the values coming from the sliders in the Graphical User Interface (GUI)
	 * updates these values in the GUIparams.java which is used by the
	 * simulation to update and set the simulation parameter. 
	 * This represent the connection between the GUI and the simulation
	 * 
	 * The sliders here allow for setting the initial or current satisfaction
	 * level of the goals as well as the preferred level of satisfaction of the
	 * goals
	 * 
	 * @param name
	 * @param val
	 */
	public void setSliderInfo(String name, double val){
				
		/*
		 * initial values of the goal fulfilment
		 */
		if (name.equalsIgnoreCase(SLinitID)) {
			GUIparams.getInstance().setInitId(val);
		}
		if (name.equalsIgnoreCase(SLinitSoc)) {
			GUIparams.getInstance().setInitSoc(val);
		}
		if (name.equalsIgnoreCase(SLinitSafe)) {
			GUIparams.getInstance().setInitSafe(val);
		}

		/*
		 * Preferred value of the goal fulfilment
		 */
		if (name.equalsIgnoreCase(SLcustoID)) {
			GUIparams.getInstance().setCustoId(val);
		}
		if (name.equalsIgnoreCase(SLcustoSoc)) {
			GUIparams.getInstance().setCustoSoc(val);
		}
		if (name.equalsIgnoreCase(SLcustoSafe)) {
			GUIparams.getInstance().setCustoSafe(val);
		}
		if (name.equalsIgnoreCase(SLcustoSubs)) {
			GUIparams.getInstance().setCustoSubs(val);
		}
	}
	
	/**
	 * This function retrieves information from the Combo-boxes as soon
	 * as a selection is changed in the Graphical User Interface (GUI). 
	 * These values are stored in the GUIparams.java which is used by the
	 * simulation to update and set the simulation parameters. 
	 * This represent the connection between the GUI and the simulation
	 * 
	 * More specifically, the combo-boxes allow for indicating the distribution
	 * of the preferred goal-level of the agents. This allows for a heterogeneous
	 * set of agents as this preference is an unique attribute of an agent.
	 * 
	 * @param name
	 * @param dist
	 */
	public void setDist(String name, String dist){
			
		if (name.equalsIgnoreCase(ComboIDDist)) {
			GUIparams.getInstance().setIdDist(dist);
		}
		if (name.equalsIgnoreCase(ComboSocDist)) {
			GUIparams.getInstance().setSocDist(dist);
		}
		if (name.equalsIgnoreCase(ComboSafeDist)) {
			GUIparams.getInstance().setSafeDist(dist);	
		}
		if (name.equalsIgnoreCase(ComboSubsDist)) {
			GUIparams.getInstance().setSubsDist(dist);
		}
	}
	
	/**
	 * Creates the listeners for the GUI-elements. 
	 * Listeners are functions that notice changes a user can 
	 * apply by (de)selecting, sliding etc. So it allows for the
	 * changes/settings inflicted by the user to have effect in the
	 * simulation
	 */
	public void createListeners(){
			
		/*
		 * used for the sliders
		 */
		d_changeListener = new ChangeListener() {
			public void stateChanged(ChangeEvent changeEvent) {
				if (changeEvent.getSource() instanceof JSlider){  // this is an instanceof abuse :(
					JSlider slider = (JSlider)changeEvent.getSource();
					if (!slider.getValueIsAdjusting()){
						double val = slider.getValue() / 10.0 ;			//needs to be a value between [0,1]
						setSliderInfo(slider.getName(), val);
					}
				}
	
			}
		};

		/*
		 * used for the Checkerbox
		 */
	   d_itemListener = new ItemListener() {
			public void itemStateChanged(ItemEvent itemEvent) {
				if (itemEvent.getSource() instanceof JCheckBox){ // instanceof abuse :(
					JCheckBox button = (JCheckBox) itemEvent.getSource();
					int state = itemEvent.getStateChange();
					if (state == ItemEvent.SELECTED)
						setGoalUsage(button.getName(), true);
					if (state == ItemEvent.DESELECTED)
						setGoalUsage(button.getName(), false);
				}
				if (itemEvent.getSource() instanceof JComboBox) { // instanceof abuse :(
					JComboBox button = (JComboBox) itemEvent.getSource();
					setDist(button.getName(), (String) button.getSelectedItem());
				}
				
			}
	   };
		
	}
	
}
