package crowdmodel.styles;

import crowdmodel.Constants;
import crowdmodel.agents.CrowdAgent;

import java.awt.Color;
import java.awt.Paint;
import java.awt.Stroke;
import java.util.Iterator;
import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.parameter.Parameters;
import repast.simphony.space.graph.Network;
import repast.simphony.util.ContextUtils;
import repast.simphony.visualization.visualization2D.style.DefaultStyle2D;

/**
* Style for crowd members in 2D displays. A distinction between crowd members can be made here in terms of visualisation.
* Use the circle shape and size specified in DefaultStyle2D. 
*
* @author Nanda Wijermans
*
*/

public class CrowdAgentStyle2D extends DefaultStyle2D{

	private Color tan = new Color(205, 133, 63);
	
	@Override
	public Paint getPaint(Object o){
		if (o instanceof CrowdAgent) {
			CrowdAgent ca = (CrowdAgent) o;
			Parameters p = RunEnvironment.getInstance().getParameters();
		
			if (ca.isDummy()){
				return getDummyColor( ca.getDummyColor());
			} else {
				if ((Boolean) p.getValue("displayBehaviour")){
					return getBehaviourColour(ca.d_myBehaviour);
				} else if ((Boolean) p.getValue("displayGoal")){
					return getDomColour(ca.getDomGoal(), ca.getCannotMove(), ca.getSatisfied());
				} else {
					return getGroupColour(ca);
				}
			}
		}
		else 
			return null;
  }

	/**
	 * Returns the heading of an agent to indicate the rotation of the 
	 * shape that is used to represent an agent on the display
	 * 
	 * @param ca
	 * @return
	 */
	public double getRotation(CrowdAgent ca){
		return ca.getHeading();
	}
	
	/**
	 * Indicates how the stroke should be painted
	 * null : don't paint the outline of the shape
	 */
	public Stroke getStroke(Object o){
		if (o instanceof CrowdAgent) {
			CrowdAgent ca = (CrowdAgent) o;
			return null;		
		}
		else 
			return null;
	}
	
	/**
	 * Returns a colour that represents the behaviour an agent shows
	 * 
	 * @param myBehaviour
	 * @return behaviour colour
	 */
	public Color getBehaviourColour(String myBehaviour){
		if (myBehaviour == Constants.B_NAME_DANCE){
			return Color.pink;
		} else if (myBehaviour == Constants.B_NAME_WALK){
			return Color.orange;
		} else if (myBehaviour == Constants.B_NAME_NOTHING) {
			return Color.green;
		} else if (myBehaviour == Constants.B_NAME_RUN) {
			return Color.cyan;
		} else if (myBehaviour == Constants.B_NAME_INHIBITED){
			return Color.LIGHT_GRAY;
		} else  // shouldn't happen
			return Color.red;
	}
	
	
	/**
	 * Returns the colour depending on what goal
	 * is dominant
	 */
	public Color getDomColour(int domGoal, boolean cannotMove, boolean satisfied){
		
		if (cannotMove)
			return Color.orange;
		if (satisfied)
			return Color.green;
		else {
			switch (domGoal){
				case Constants.GOAL_IDENTITY : return Color.yellow;
				case Constants.GOAL_SOCIAL : return Color.magenta;
				case Constants.GOAL_SAFETY : return Color.red;
				case Constants.GOAL_SUBSISTENCE : return tan;
				default : return Color.blue; //Syst.err
			}
		}
	}
	
	/**
	 * Returns the colour depending on the group it belongs to
	 * in the sense that group members have the same colour
	 * 
	 */
	public Color getGroupColour(CrowdAgent ca){
		Context<CrowdAgent> agentcontext = ContextUtils.getContext(ca);
		Network<CrowdAgent> socnet = (Network<CrowdAgent>) agentcontext.getProjection("Social Network");
		Iterator<CrowdAgent> mygroup = socnet.getAdjacent(ca).iterator();
		int sumIDs = ca.getAgentID()^2;
		int counter = 1;
		while(mygroup.hasNext()){
			sumIDs += mygroup.next().getAgentID()^2;	
			++counter;
		}
		
		int R = (sumIDs > 255 ? sumIDs % 255 : sumIDs);
		int G = 255;
		int B = 255;
		switch (counter) {
		case 1: {
				G = 255;
				B = 255;
			break;}
		case 2: {
			G = 255;
			B = 0;
		break;}
		case 3: {
			G = 0;
			B = 0;
		break;}
		default: G = 255 / counter;
			break;
		}
		
		
		Color groupColour = new Color(R, B, B); 
		return groupColour;	
	}
	
	
	public Color getDummyColor(int colorNr){

		switch (colorNr) {
		case 1: {
			return Color.blue;}
		case 2: {
			return Color.darkGray;}
		case 3: {
			return Color.lightGray;}
		default:
			return Color.white; // syst.err
		}

	}

	
}
