package crowdmodel.test;

import java.util.HashMap;
import java.util.Iterator;

import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunState;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.grid.GridPoint;
import saf.core.ui.actions.MaxAction;
import crowdmodel.CObject;
import crowdmodel.Constants;
import crowdmodel.agents.BehaviourRule;
import crowdmodel.agents.CrowdAgent;
import crowdmodel.utils.MathUtils;
import crowdmodel.utils.Misc;
import crowdmodel.utils.MiscSingleton;

public class BehaveTest extends CObject {

	CrowdAgent d_testDummy;
	
	int d_crowdsize;
	Iterator<CrowdAgent> d_agents;
	
	double[][] d_distanceMatrix;
	HashMap<Integer, CrowdAgent> d_index2agent;
	HashMap<CrowdAgent, Integer> d_agent2index;
	
	
	public BehaveTest(CrowdAgent agent, int crowdsize, Iterator<CrowdAgent> agents)  {
		d_id = MiscSingleton.getInstance().nextID();
		d_testDummy = agent;
		
		d_crowdsize = crowdsize;
		d_agents = agents;
		d_index2agent = new HashMap<Integer, CrowdAgent>(crowdsize);
		d_agent2index = new HashMap<CrowdAgent, Integer>(crowdsize);
		
		initDistanceMatrix();	
		
	}

	
@ScheduledMethod(start = 1, interval = 1, priority = 4.0) // high priority = high number, earlier ; low priority = low number, later
	public void testIt(){
		//System.out.println("BehaveTest.testIt(): we gaan testen!!");
//		testNearestAgent();
	
	 	if(d_testDummy != null){   //meaning that there are no agents!
	 		testBselection();
	 	}
		
	}
	
	

/**
 *  Initialises the distance matrix and the variables 
 *  that are involved with it
 *  
 */
public void initDistanceMatrix(){
	//System.out.println("GroupLevel.initDistanceMatrix: init matrixxxxie");
	d_distanceMatrix = new double[d_crowdsize][d_crowdsize];
	for (int i = 0; i < d_crowdsize; ++i) {
		for (int j = 0; j < d_crowdsize; ++j){
			d_distanceMatrix[i][j] = 0;
		}
		if (d_agents.hasNext()){
			CrowdAgent ca = (CrowdAgent) d_agents.next();
			d_index2agent.put(i,ca);
			d_agent2index.put(ca, i);
		} else {
			System.out.println("GroupLevel.initDistanceMatrix: aantal agents is minder dan de matrix lengte, da ken nie hoor!");
		}
	}
	
}

/**
 * Sets the distances between every agent in the network
 * This way we can check whether the calculation of the nearest
 * agent is performing correctly.
 * 
 */
//@ScheduledMethod(start = 0, interval = 1, priority = 9)  // high priority = high number; low priority = low number
	public void updateDistanceMatrix(){
		//System.out.println("BehaveTest.initDistanceMatrix: update matrixxxxie");
		Context world = RunState.getInstance().getMasterContext();
		//Context agentContext = world.getSubContext(Constants.ID_AGENT_CONTEXT);
		ContinuousSpace space = (ContinuousSpace) world.getProjection("Continuous Space");
	
		for (int i = 0; i < d_crowdsize; ++i) {
			NdPoint point1 = space.getLocation(d_index2agent.get(i));
			for (int j = 1; j < d_crowdsize; ++j){
				NdPoint point2 = space.getLocation(d_index2agent.get(j));
				double dist = space.getDistance(point1, point2);	
				d_distanceMatrix[i][j] = dist;
				d_distanceMatrix[j][i] = dist;
			}
		}	
		//Misc.matrixPrint(d_distanceMatrix); //prints the matrix in the console
	}

	/**
	 * Function that checks whether an agent will head towards
	 * the nearest agent. This is done in two ways:
	 * 1) the distance found is also found by the test (matrix of distances between agents)
	 * 2) whether the nearestAgent found by the matrix matches with the one the agent has found.
	 * 
	 */

	public void testNearestAgent(){
		double dist = d_testDummy.getDistNearestAgent();
		CrowdAgent nearest4dummy = d_testDummy.getNearestAgent();
		
		int x = d_agent2index.get(d_testDummy);
		int y = d_agent2index.get(nearest4dummy);
		double distMatrix = d_distanceMatrix[x][y];
		
		/* checks whether the correct distance is calculated by testDummy */
		if (dist != distMatrix){
			System.out.println("BehaveTest.testNearestAgent : dist nearest agent !=  matrix val of the dist to this person - " + dist + " != " + distMatrix);
		}
		
		double mindist = Constants.WORLD_MAX_DIST;
		CrowdAgent nearest4matrix = null;
		for (int i = 0; i < d_crowdsize; i++) {
			if (i != x){
				double curdist = d_distanceMatrix[x][i];
				if (curdist < mindist){
					mindist = curdist;
					nearest4matrix = d_index2agent.get(i);
				}
			}
		}
		
		/* checks whether the correct nearest agent is used by testDummy */
		if (nearest4dummy != nearest4matrix){
			System.out.println("BehaveTest.testNearestAgent() : nearestAgent matched niet!");
		}
	}
	
	
	
	public void testHeading2Point(){
	// test 8 condities : 4 kwadranten, de 4 assen op het kwadrant of de juiste repast hoek berekend word
	// rekening houdend met het assenstelsel van repast (360/0 90 180/-180 -90)
	
	NdPoint me = new NdPoint(0,0,0);
	NdPoint q1 = new NdPoint(4,3,0);
	NdPoint q2 = new NdPoint(4,-3,0);
	NdPoint q3 = new NdPoint(-4,-3,0);
	NdPoint q4 = new NdPoint(-4,3,0);
	
	NdPoint q1q2 = new NdPoint(4,0,0);  // on the positive side of the x-as; same vertical height as me/origin
	NdPoint q2q3 = new NdPoint(0,-3,0); // on the negative side of the y-as; same horizontal height as me/origin
	NdPoint q3q4 = new NdPoint(-4,0,0);  // on the negative side of the x-as; same vertical height as me/origin
	NdPoint q4q1 = new NdPoint(0,3,0);  // on the positive side of the x-as; same horizontal height as me/origin
	
	}
	
	public void testBselection(){
		String b = d_testDummy.d_myBehaviour;
		BehaviourRule brule = d_testDummy.d_memory.getBehaviour(b);
		
		
		if (b != "inhibited" && b != "nothing") {
			BehaviourRule walk = d_testDummy.d_memory.getBehaviour(Constants.B_NAME_WALK);
			BehaviourRule run = d_testDummy.d_memory.getBehaviour(Constants.B_NAME_RUN);
			BehaviourRule dance = d_testDummy.d_memory.getBehaviour(Constants.B_NAME_DANCE);
			
			String bMaxActivation = Constants.B_NAME_WALK;
			double activ = walk.getActivation();
			double activR = run.getActivation();
			double activD = dance.getActivation();
			
//			if (activ == activR || activR == activD){
//				System.out.println("BehaveTest.bSelectTest(): activation values of the behaviours are identical");
//			}
//			if (activ < activR){
//				activ = activR;
//				bMaxActivation = Constants.B_NAME_RUN;
//			}
//			if (activ < activD){
//				activ = activD;
//				bMaxActivation = Constants.B_NAME_DANCE;
//			}
			
			String bMaxUtil = Constants.B_NAME_WALK;
			double utilW = d_testDummy.calcUtil(walk);
			double utilR = d_testDummy.calcUtil(run);
			double utilD = d_testDummy.calcUtil(dance);
			
			double maxUtil = utilW;
			
			if (maxUtil < utilR){
				bMaxUtil = Constants.B_NAME_RUN;
				maxUtil = utilR;
			}
			if (maxUtil < utilD){
				bMaxUtil = Constants.B_NAME_DANCE;
				maxUtil = utilD;
			}
			if (utilW == utilR || utilR == utilW){
				//System.out.println("BehaveTest.testBselection(): utils values of the behaviours are identical");
			}
			
			if (b != bMaxActivation && b != bMaxUtil){
				//System.out.println("BehaveTest.testBselection(): "+ b + " is chosen, however not most active or with the highest util");
			}
			
			double bUtil = d_testDummy.calcUtil(brule);
			BehaviourRule bruleMaxActivation = d_testDummy.d_memory.getBehaviour(bMaxActivation);
			double utilMaxActiv = d_testDummy.calcUtil(bruleMaxActivation);
			if (bUtil < utilMaxActiv){
				//System.out.println("BehaveTest.testBselection(): utility of the most active behaviour is higher than the behaviour chosen, that shouldn't happen");
			}
		}
	}
	
	
}
