package crowdmodel.utils;

import crowdmodel.Constants;
import repast.simphony.parameter.Parameters;

/**
 * This class contains all variables that are used for
 * plotting, measuring, manipulation, etc.
 * 
 * In this model we hava a multi-level approach, which means
 * that we measure on multiple levels: 
 * 	- inter-group : between groups (this is not included yet)
 *  - intra-group : within groups 
 *  - inter-individual: between individuals
 *  - intra-individual: within an individual
 * 
 * In Repast data sets are build based upon them, so that one
 * can easily include or exclude variables/functions for
 * plotting or output means
 * 
 * @author Nanda Wijermans
 *
 */
public class DataSets {

	/*
	 * Group level variables 
	 * 
	 * Note that group level variables are always constellations of a set individuals.
	 * They are always a representation or label for a set individual. For instance
	 * density : is a description on group level that results in a number of agents in a certain area
	 * cluster : a temporary label of a set of agents that share a certain characteristic, such as walking, having a high arousal level, etc.
	 */
	
	
	/*
	 *  For initialisation
	 * 
	 */
	int gCrowdsize;		// number of agents [0 - ??] //N: todo koppelen aan density en vise versa
	int gDensity;    	// density : number of people per m^2 [0 - 7] //N: todo max rho uitzoeken
	int gNrLeaders;		// number of leaders {0|1|2}
	int gPeaceLeaders;	// percentage of leaders that is peaceful [0 - 100] //N: todo koppelen aan de val van warLeaders
	int gWarLeaders;		// percentage of leaders that is warful [0 - 100]
	
	/* 
	 * Goal preferences  
	 * custo = all the same pre- or user defined value [0 - 1]
	 * dist = distribution {normal|gamma|...??}
	 */
	boolean gPref_custo;		// indicates whether the goal preferences are a pre- or user defined value OR following a distribution

	double gIdentity_pref_custo;
	double gSocial_pref_custo;
	double gSafety_pref_custo;
	double gSubsistence_pref_custo;
	
	String gIdentity_pref_dist;
	String gSocial_pref_dist;
	String gSafety_pref_dist;
	String gSubsistence_pref_dist;
	
	/*
	 * Goal satisfaction settings
	 * custo = all the same pre- or user defined value [0 - 1]
	 * dist = distribution {normal|gamma| uniform ...??}
	 */
	boolean gSatis_custo;		// indicates whether the goal satisfaction are a pre- or user defined value OR following a distribution

	double gIdentity_satis_custo;
	double gSocial_satis_custo;
	double gSafety_satis_custo;
	double gSubsistence_satis_custo;
	
	String gIdentity_satis_dist;
	String gSocial_satis_dist;
	String gSafety_satis_dist;
	String gSubsistence_satis_dist;

	/*
	 * Arousal level settings
	 * custo = all the same pre- or user defined value [Low | Medium | High]
	 * otherwise a percentage is used to make the difference
	 */
	
    boolean gArousal_custo; 	// indicates whether arousal is a pre- or user defined value OR following a distribution

    int gLowArousal;			// percentage of agents with a Low arousal level
    int gMediumArousal;			// percentage of agents with a Medium arousal level
    int gHighArousal;			// percentage of agents with a High arousal level
    
	
    /*
     * For plotting/outputting
     * 
     */
    
    /* 
     * Goal dominance
     * Amount of individuals given time t
     */
    int d_identity_dominance;
    int d_social_dominance;
    int d_safety_dominance;
    int d_subsistence_dominance;
    

	int d_move_id = 0;
	int d_move_social = 0;
	int d_move_safety = 0;
	int d_move_subs = 0;
	
	/*
	 * Density perception
	 */
	int d_HlocalDens;
	int d_MlocalDens;
	int d_LlocalDens;
	
    /**
     * Constructor for a dataSet
     * Depending on the user-input the values are initialised
     * with a default or user-defined value
     * 
     * @param p  (the user defined parameters)
     */
    public DataSets(Parameters p){
    	defaultInit();
    	resetCounters();
    }

    /**
     * Default values for initialisation of the agents
     * These are described in group-level terms,
     * to be translated to individual/agent values when
     * an agent is created
     */
 
    private void defaultInit(){
    	gCrowdsize = Constants.DEFAULT_CROWDSIZE;	
    	gDensity = Constants.DEFAULT_DENSITY;   
    	gNrLeaders = Constants.DEFAULT_NR_LEADERS;		
    	gPeaceLeaders = Constants.DEFAULT_PEACE_LEADERS;	
    	gWarLeaders = Constants.DEFAULT_WAR_LEADERS;
    	    	
    	gPref_custo = true;	
    	gIdentity_pref_custo = 0.7;
    	gSocial_pref_custo = 0.8;
    	gSafety_pref_custo = 0.1;
    	gSubsistence_pref_custo = 0.6;
    	
    	gIdentity_pref_dist = "normal";
    	gSocial_pref_dist = "normal";
    	gSafety_pref_dist = "normal";
    	gSubsistence_pref_dist = "normal";
    	
    	gSatis_custo = true;
    	gIdentity_satis_custo = 0.2;
    	gSocial_satis_custo = 0.1;
    	gSafety_satis_custo = 1;
    	gSubsistence_satis_custo = 1.0;
    	
    	gIdentity_satis_dist = "normal";
    	gSocial_satis_dist = "normal";
    	gSafety_satis_dist = "normal";
    	gSubsistence_satis_dist = "normal";
    	
    	gArousal_custo = true;
    	gLowArousal = 0;			
        gMediumArousal = 80;			
        gHighArousal = 20;	
    }

    public void resetCounters(){
    	 d_identity_dominance = 0;
    	 d_social_dominance = 0;
    	 d_safety_dominance = 0;
    	 d_subsistence_dominance = 0;
    	 
    	 d_move_id = 0;
    	 d_move_social = 0;
    	 d_move_safety = 0;
    	 d_move_subs = 0;
    
    	 d_HlocalDens = 0;
    	 d_MlocalDens = 0;
    	 d_LlocalDens = 0;
    }
    
    public void incIdentityDom(){
    	++ d_identity_dominance;
    }
    public void incSocialDom(){
    	++ d_social_dominance;
    }
    public void incSafetyDom(){
    	++ d_safety_dominance;
    }
    public void incSubsistenceDom(){
    	++ d_subsistence_dominance;
    }    
    
    public void incMoveID(){
    	++ d_move_id;
    }

    public void incMoveSoc(){
    	++ d_move_social;
    }
   public void incMoveSafe(){
    	++ d_move_safety;
    }
    public void d_moveSubs(){
    	++ d_move_subs;
    } 
}
